package com.share.help.controller;

import com.share.help.Constants;
import com.share.help.entity.custorm.*;
import com.share.help.form.*;
import com.share.help.res.DefaultRes;
import com.share.help.res.JSONResponse;
import com.share.help.res.Page;
import com.share.help.res.Result;
import com.share.help.service.ActivityService;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(Constants.ACTIVITY_INTERFACE)
public class ActivityController {

    @Autowired
    private ActivityService activityService;

    private Logger logger= LoggerFactory.getLogger(ActivityController.class);

    /**
     * 发布求助信息
     * @return 返回发布结果
     */
    @PutMapping(Constants.ACTIVITY_INTERFACE_SEND_HELP)
    public JSONResponse<DefaultRes,Void> send(SendHelpForm sendHelpForm,@RequestParam(value = "friendUserId[]",required = false) String []friendUserId,@CookieValue(Constants.USER_COOKIE) String userId){
            if(StringUtils.isNoneEmpty(userId,sendHelpForm.getTitle(),sendHelpForm.getContent(),sendHelpForm.getServiceAddress())&&
                    ObjectUtils.allNotNull(sendHelpForm.getActivityStartTime(),sendHelpForm.getActivityEndTime()
                            ,sendHelpForm.getActivityImgFile())){
                sendHelpForm.setUserId(userId);
                return activityService.sendHelp(sendHelpForm,friendUserId);
            }else{
                return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
            }
    }

    /**
     * 返回活动列表数据
     * @param findActivityForm 查询表单
     * @return 返回活动列表数据
     */
    @GetMapping(Constants.ACTIVITY_INTERFACE_FIND)
    public JSONResponse<FindActivityRes.Res,FindActivityRes> find(FindActivityForm findActivityForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(findActivityForm.getCurrentPage()!=null&StringUtils.isNotEmpty(userId)){
            findActivityForm.setUserId(userId);
            return activityService.find(findActivityForm);
        }else{
            return new JSONResponse<>(FindActivityRes.Res.fail,Result.FAIL);
        }
    }

    /**
     * 查看活动详情
     * @param activityId 活动id
     * @return 返回活动列表
     */
    @GetMapping(Constants.ACTIVITY_INTERFACE_FIND_ID)
    public JSONResponse<DefaultRes, ActivityDetail> find(@PathVariable Long activityId,@CookieValue(Constants.USER_COOKIE) String userId){
        if(activityId==null||StringUtils.isEmpty(userId)){
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }else{
            return activityService.find(activityId,userId);
        }
    }

    /**
     * 报名活动
     * @return 返回报名结果
     */
    @PostMapping(Constants.ACTIVITY_INTERFACE_APPLY)
    public JSONResponse<DefaultRes,Integer> apply(UpdateActivityForm updateActivityForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(StringUtils.isNotEmpty(userId)&&ObjectUtils.allNotNull(updateActivityForm.getActivityId(),updateActivityForm.getActivityStatus())){
            updateActivityForm.setUserId(userId);
            try {
                return activityService.apply(updateActivityForm);
            } catch (RuntimeException e) {
                e.printStackTrace();
                logger.error(e.toString());
                return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
            }
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 统计求助信息数据
     * @return 返回统计结果
     */
    @GetMapping(Constants.ACTIVITY_INTERFACE_SEEK_HELP)
    public JSONResponse<DefaultRes, Page<List<SeekHelpSum>>> seekHelp(SeekHelpForm seekHelpForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(userId==null||seekHelpForm.getCurrentPage()==null){
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }else{
            seekHelpForm.setUserId(userId);
            return activityService.seekHelpCount(seekHelpForm);
        }
    }

    /**
     *
     * @param findUserHelp 查找活动志愿者列表
     * @return 返回志愿者列表
     */
    @GetMapping(Constants.ACTIVITY_INTERFACE_SEEK_HELP_USER)
    public JSONResponse<DefaultRes,List<UserWithName>> seekHelpUser(FindUserHelp findUserHelp){
        if(ObjectUtils.allNotNull(findUserHelp.getActivityId(),findUserHelp.getActivityStatus())){
            return activityService.seekHelpList(findUserHelp);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 查找帮助列表
     * @param queryHelpForm 查询表单
     * @param userId 用户id
     * @return 返回帮助信息
     */
    @GetMapping(Constants.ACTIVITY_INTERFACE_HELP)
    public JSONResponse<DefaultRes, Page<List<HelpRes>>> queryHelp(QueryHelpForm queryHelpForm,@CookieValue(Constants.USER_COOKIE) String userId){
        if(ObjectUtils.allNotNull(queryHelpForm.getCurrentPage(),userId)){
            queryHelpForm.setUserId(userId);
            return activityService.queryHelp(queryHelpForm);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 评价活动
     * @param helpScoreForm 评价表单
     * @return 返回评价结果
     */
    @PostMapping(Constants.ACTIVITY_INTERFACE_SCORE)
    public JSONResponse<DefaultRes,Void> updateScore(HelpScoreForm helpScoreForm){
        if(ObjectUtils.allNotNull(helpScoreForm.getHistoryId(),helpScoreForm.getScore())){
            return activityService.updateScore(helpScoreForm);
        }else {
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }
}
