package com.share.help.dao;

import com.share.help.entity.UserEntity;
import com.share.help.entity.custorm.FindUserRes;
import com.share.help.entity.custorm.UserWithName;
import com.share.help.res.Page;
import com.share.help.res.account.QueryRes;
import org.apache.ibatis.annotations.*;

import java.util.List;

/**
 * 用户数据接口
 */
public interface UserMapper {

    /**
     * 添加用户
     * @param user 用户信息
     */
    @Insert("insert into user(user_id,password,name,sex,head_img,address,service_address,user_type,mobile," +
            "email,info,time_score,chat_status,age)value(#{userId},#{password},#{name},#{sex}," +
            "#{headImg},#{address},#{serviceAddress},#{userType},#{mobile},#{email},#{info}," +
            "#{timeScore},#{chatStatus},#{age})")
    boolean insert(UserEntity user);

    /**
     * 根据用户账号删除用户
     * @param userId 用户账号
     */
    @Delete("delete from user where user_id=#{userId}")
    boolean delete(@Param("userId") String userId);

    /**
     * 根据用户账号查找用户
     * @param userId 用户id
     * @return 返回用户
     */
    @Select("select * from user where user_id=#{userId}")
    @ResultMap("userMap")
    UserEntity findOne(@Param("userId") String userId);

    /**
     * 根据用户账号密码查找用户
     * @param userId 用户id
     * @param password 密码
     * @return 返回用户
     */
    @Select("select * from user where user_id=#{userId} and password=#{password}")
    @ResultMap("userMap")
    UserEntity findOneWithPwd(@Param("userId") String userId, @Param("password") String password);

    /**
     *
     * @return 返回更新结果
     */
    boolean update(UserEntity userEntity);

    /**
     * 根据用户id判断用户是否存在
     * @param userId 用户id
     * @return true存在，false不存在
     */
    @Select("select count(*) from user where user_id=#{userId}")
    boolean hasUser(@Param("userId") String userId);
    /**
     *
     * @return 获取所有用户
     */
    @Select("select * from user")
    @ResultMap("userMap")
    List<UserEntity> getAll();

    /**
     * 分页查询用户
     * @param start 起始索引
     * @param count 查询个数
     * @return 返回用户列表
     */
    List<UserEntity> getPage(@Param("start") Integer start, @Param("count") Integer count,
                             @Param("userTypes") List<String> userTypes,@Param("info") String info,@Param("serviceAddress") String serviceAddress);

    /**
     * 获取志愿者分页数
     * @param pageSize 分页大小
     * @return 返回分页信息
     */
    FindUserRes count(@Param("pageSize") Integer pageSize, @Param("userTypes") List<String> userTypes,@Param("info")
            String info,@Param("serviceAddress") String serviceAddress);

    Page<List<UserEntity>> countUserByManager(@Param("pageSize") Integer pageSize, @Param("userTypes") List<String> userTypes,@Param("info")
            String info,@Param("serviceAddress") String serviceAddress);

    /**
     * 查找用户
     */
    List<UserWithName> queryUser(@Param("name") String name, @Param("info") String info,@Param("userId") String userId,@Param("targetUserId") String targetUserId);


    Page<List<UserWithName>> countUser(@Param("name") String name, @Param("info") String info, @Param("userId") String userId,@Param("targetUserId") String targetUserId, @Param("pageSize") Integer pageSize);

    /**
     * 查看用户信息
     * @param targetUserId 被查看用户id
     * @param userId 查看用户id
     * @return 返回用户信息
     */
    QueryRes findInfo(@Param("targetUserId") String targetUserId,@Param("userId") String userId);
}
