package com.share.help.service;

import com.share.help.dao.ActivityHistoryMapper;
import com.share.help.dao.ActivityMapper;
import com.share.help.dao.ManagerMapper;
import com.share.help.dao.UserMapper;
import com.share.help.entity.ManagerEntity;
import com.share.help.entity.UserEntity;
import com.share.help.form.*;
import com.share.help.form.manager.FindActivityForm;
import com.share.help.form.manager.UpdateActivityForm;
import com.share.help.res.*;
import com.share.help.res.account.ManagerLoginRes;
import com.share.help.res.account.RegisterRes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.List;

@Service
public class ManagerService {

    @Autowired
    private ManagerMapper managerMapper;

    private Logger logger= LoggerFactory.getLogger(ManagerService.class);

    @Value("${page-size.manager}")
    private int pageSize;

    //用户分页配置
    @Value("${page-size.manager}")
    private int userPageSize;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private ActivityMapper activityMapper;

    @Autowired
    private ImageService imageService;

    @Autowired
    private ActivityHistoryMapper activityHistoryMapper;

    /**
     * 登录
     * @param managerLoginForm 登录表单
     * @return 返回登录结果
     */
    public JSONResponse<ManagerLoginRes.Res, ManagerLoginRes> login(ManagerLoginForm managerLoginForm) {
        ManagerEntity managerEntity=managerMapper.findUser(managerLoginForm.getManagerId());
        if (managerEntity==null){
            logger.error("管理员账号："+ managerLoginForm.getManagerId()+"不存在");
            return new JSONResponse<>(ManagerLoginRes.Res.valid_fail, Result.FAIL);
        }
        String password= DigestUtils.md5DigestAsHex(managerLoginForm.getPassword().getBytes());
        if(password.equals(managerEntity.getPassword())){
           return new JSONResponse<ManagerLoginRes.Res,
                   ManagerLoginRes>(ManagerLoginRes.Res.ok,Result.OK).setBody(new ManagerLoginRes(managerEntity));
        }else{
            logger.error("管理员:"+managerEntity.getManagerId()+"密码验证错误");
            return new JSONResponse<>(ManagerLoginRes.Res.valid_fail, Result.FAIL);
        }
    }

    /**
     * 注册
     * @param managerRegisterForm 注册表单
     * @return 返回注册结果
     */
    public JSONResponse<RegisterRes, Void> register(ManagerRegisterForm managerRegisterForm) {
        if(managerMapper.hasUser(managerRegisterForm.getManagerId())){
            return new JSONResponse<>(RegisterRes.user_repeat,Result.FAIL);
        }
        ManagerEntity managerEntity=new ManagerEntity();
        managerEntity.setManagerId(managerRegisterForm.getManagerId());
        managerEntity.setPassword(DigestUtils.md5DigestAsHex(managerRegisterForm.getPassword().getBytes()));
        if(managerMapper.insert(managerEntity)){
            return new JSONResponse<>(RegisterRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(RegisterRes.fail,Result.FAIL);
        }
    }

    /**
     * 管理员列表
     * @param queryManager 查询表单
     * @return 返回管理员列表
     */
    public JSONResponse<DefaultRes, Page<List<ManagerRes>>> list(QueryManager queryManager) {
        Page<List<ManagerRes>> page=managerMapper.count(pageSize);
        List<ManagerRes> list=managerMapper.list((queryManager.getCurrentPage()-1)*pageSize,pageSize);
        page.setBody(list);
        page.setCurrentPage(queryManager.getCurrentPage());
        return new JSONResponse<DefaultRes, Page<List<ManagerRes>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 更新管理员
     * @param manager 管理员表单
     * @return 返回更新结果
     */
    public JSONResponse<DefaultRes,Void> update(UpdateManager manager){
         ManagerEntity managerEntity=managerMapper.findUser(manager.getManagerId());
         managerEntity.setPassword(DigestUtils.md5DigestAsHex(manager.getPassword().getBytes()));
         if(managerMapper.update(managerEntity)){
             return new JSONResponse<>(DefaultRes.ok,Result.OK);
         }else{
             return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
         }
    }

    /**
     * 删除管理员
     * @param managerId 管理员id
     * @return 返回删除结果
     */
    public JSONResponse<DefaultRes, Void> delete(String managerId) {
        if (managerMapper.delete(managerId)) {
            return new JSONResponse<>(DefaultRes.ok, Result.OK);
        } else {
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }


    /**
     *
     * @param findUserForm 查找用户表单
     * @return 返回用户列表
     */
    public JSONResponse<DefaultRes,Page<List<UserEntity>>> find(FindUserForm findUserForm){
        Page<List<UserEntity>> page=userMapper.countUserByManager(userPageSize,findUserForm.getUserTypes(),findUserForm.getInfo(),findUserForm.getServiceAddress());
        page.setCurrentPage(findUserForm.getCurrentPage());
        List<UserEntity> userEntityList=userMapper.getPage((findUserForm.getCurrentPage()-1)* userPageSize, userPageSize,findUserForm.getUserTypes(),
                findUserForm.getInfo(),findUserForm.getServiceAddress());
        page.setBody(userEntityList);
        return new JSONResponse<DefaultRes,Page<List<UserEntity>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     *
     * @param findActivityForm 查找活动列表
     * @return 返回活动列表
     */
    public JSONResponse<DefaultRes, Page<List<ManagerActivityRes>>> findActivity(FindActivityForm findActivityForm) {
        Page<List<ManagerActivityRes>> page=activityMapper.countActivity(pageSize);
        page.setCurrentPage(findActivityForm.getCurrentPage());
        page.setBody(activityMapper.queryActivity((findActivityForm.getCurrentPage()-1)*pageSize,pageSize));
        return new JSONResponse<DefaultRes, Page<List<ManagerActivityRes>>>(DefaultRes.ok,Result.OK).setBody(page);
    }

    /**
     * 更新活动信息
     * @param updateActivityForm 更新表单
     * @return 返回更新结果
     */
    public JSONResponse<DefaultRes, Void> updateActivity(UpdateActivityForm updateActivityForm) {

        String imageName = null;
        if(updateActivityForm.getActivityImgFile()!=null) {
            //保存活动背景图
            imageName = imageService.saveImg(updateActivityForm.getActivityImgFile());
            if (imageName == null) {
                logger.error("活动背景图保存失败");
                return new JSONResponse<>(DefaultRes.fail, Result.FAIL);
            }
        }

        if(activityMapper.updateActivity(updateActivityForm.getActivityId(),updateActivityForm.getTitle(),updateActivityForm.getContent(),imageName)){
            return new JSONResponse<>(DefaultRes.ok,Result.OK);
        }else{
            return new JSONResponse<>(DefaultRes.fail,Result.FAIL);
        }
    }

    /**
     * 查询时间币
     * @param queryScoreForm 查询时间币表单
     * @return 返回时间币
     */
    public JSONResponse<DefaultRes, TimeScore> queryTimeScore(QueryScoreForm queryScoreForm) {
        TimeScore timeScore=activityHistoryMapper.queryActivityHistory(queryScoreForm.getUserId(),queryScoreForm.getActivityId());
        return new JSONResponse<DefaultRes, TimeScore>(DefaultRes.ok,Result.OK).setBody(timeScore);
    }
}
